#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <vector>
#include <unordered_map>
#include <unordered_set>
#include <algorithm>
#include <stack>

using namespace std;

using ll = long long;

const int mod = 998244353;


template<class T>
T sqr (T x)
{
    return x * x;
}


ll powmod (ll x, int y)
{
    return y ? (y & 1 ? x * powmod (x, y - 1) : sqr (powmod (x, y / 2))) % mod : 1;
}


ll fact (int n)
{
    static vector<int> dp = { 1 };

    if (n >= dp.size ())
    {
        fact (n - 1);

        dp.push_back ((fact (n - 1) * n) % mod);
    }

    return dp[n];
}


ll ofact (int n)
{
    static vector<int> dp = { 1 };

    if (n >= dp.size ())
    {
        ofact (n - 1);

        dp.push_back ((ofact (n - 1) * powmod (n, mod - 2)) % mod);
    }

    return dp[n];
}


ll C (int n, int k)
{
    if (k < 0 || k > n)
        return 0;

    return ((fact (n) * ofact (k) % mod) * ofact (n - k)) % mod;
}


void dfs (const vector<vector<int>> &gr, const vector<int> &lang, vector<int> &used,
    int v, int col, int mask)
{
    if (used[v] != -1 || ((1 << lang[v]) & mask) == 0)
        return;

    used[v] = col;

    for (auto it : gr[v])
        dfs (gr, lang, used, it, col, mask);
}


void grsum (const vector<vector<int>> &gr, const vector<int> &lang,
    vector<vector<int>> &dprime, int mask)
{
    vector<int> col (gr.size (), -1);

    for (int i = 0; i < gr.size (); i++)
        dfs (gr, lang, col, i, i, mask);

    vector<int> cnt (gr.size ());

    for (int i = 0; i < col.size (); i++)
        if (col[i] != -1)
            cnt[col[i]]++;

    for (auto it : cnt)
        if (it != 0)
        {
            for (int i = 1; i <= it; i++)
            {
                dprime[i][mask] += C (it, i);

                if (dprime[i][mask] >= mod)
                    dprime[i][mask] -= mod;
            }
        }
}


void f (vector<int> &d, int n)
{
    for (int i = 0; i < n; ++i)
        for (int mask = 0; mask < (1 << n); ++mask)
            if ((mask >> i) & 1)
            {
                d[mask] -= d[mask ^ (1 << i)];

                if (d[mask] < 0)
                    d[mask] += mod;
            }
}


int f1 (int n)
{
    static vector<int> dp = { 1 };

    if (n >= dp.size ())
    {
        f1 (n - 1);
        dp.push_back (0);

        for (int k = 1; k <= n; k++)
        {
            dp.back () += (C (n - 1, k - 1) * f1 (n - k)) % mod;
            if (dp.back () >= mod)
                dp.back () -= mod;
        }
    }

    return dp[n];
}


int pcnt (int mask)
{
    return mask ? 1 + pcnt (mask & (mask - 1)) : 0;
}


void solve (istream &cin = std::cin, ostream &cout = std::cout)
{
    int n, k;

    cin >> n >> k;

    vector<int> arr (n);

    for (auto &it : arr)
    {
        cin >> it;
        it--;
    }

    vector<vector<int>> gr (n);

    for (int i = 0; i < gr.size (); i++)
    {
        int a, b;

        cin >> a >> b;
        a--, b--;

        gr[a].push_back (b);
        gr[b].push_back (a);
    }

    vector<vector<int>> dprime (n + 1, vector<int> (1ll << k));

    for (int i = 0; i < (1 << k); i++)
        grsum (gr, arr, dprime, i);

    for (auto &row : dprime)
    {
        //reverse (row.begin (), row.end ());

        f (row, k);
    }

    long long ans = 0;

    for (int i = 1; i <= n; i++)
        for (int p = 0; p < (1 << k); p++)
        {
            int q = pcnt (p);

            ans += ((dprime[i][p] * f1 (n - i)) % mod) * ((2 << k) - (2 << (k - q)));
        }

    cout << ans % mod << endl;
}

int main ()
{
    ios_base::sync_with_stdio (false);
    cin.tie (nullptr);

    cout << fixed << setprecision (20);

#ifdef LOCAL
    ifstream fin ("input.txt");

    solve (fin);

    cout << setprecision (4);
    cout << "clock: " << clock () / (double)CLOCKS_PER_SEC << endl;
#else
    solve ();
#endif

    return 0;
}
