#include <iostream>
#include <vector>
#include <set>

using namespace std;

const int MAXK = 1010;

long long hashes1[2 * MAXK];
long long powers1[2 * MAXK];
long long hashes2[2 * MAXK];
long long powers2[2 * MAXK];


long long POW1 = 53;
long long MOD1 = (long long)1e9 + 7;
long long POW2 = 29;
long long MOD2 = (long long)1e9 + 9;


void make_hashes(const string &str) {
  hashes1[0] = 0;
  hashes2[0] = 0;
  for (size_t i = 0; i < str.size(); ++i) {
    hashes1[i + 1] = (hashes1[i] * POW1 + (str[i] - 'a' + 1)) % MOD1;
    hashes2[i + 1] = (hashes2[i] * POW2 + (str[i] - 'a' + 1)) % MOD2;
  }

  powers1[0] = 1;
  powers2[0] = 1;
  for (int i = 1; i < 2 * MAXK; ++i) {
    powers1[i] = (powers1[i - 1] * POW1) % MOD1;
    powers2[i] = (powers2[i - 1] * POW2) % MOD2;
  }
}

long long get_hash(int l, int r) {
  long long left = (hashes1[r + 1] + MOD1 - (hashes1[l] * powers1[r - l + 1]) % MOD1) % MOD1;
  long long right = (hashes2[r + 1] + MOD2 - (hashes2[l] * powers2[r - l + 1]) % MOD2) % MOD2;
  return (right << 32ll) + left;
}

int main() {
  string s;
  getline(cin, s);
  long long n;
  cin >> n;
  int k = (int)s.size();
  long long ans = 0;

  make_hashes(s + s);

  set<long long> cur_hashes;
  vector<int> is_uniq;
  for (int len = 1; len <= k; ++len) {
    is_uniq.clear();
    cur_hashes.clear();
    for (int pos = 0; pos + len - 1 < n && pos < k; ++pos) {
      long long cur_hash = get_hash(pos, pos + len - 1);
      if (cur_hashes.find(cur_hash) == cur_hashes.end()) {
        is_uniq.push_back(1);
        ++ans;
        cur_hashes.insert(cur_hash);
      } else {
        is_uniq.push_back(0);
      }
    }
  }

  if (n <= k) {
    cout << ans << endl;
    return 0;
  }

  vector<long long> pref_sums(is_uniq.size() + 1);
  pref_sums[0] = 0;
  for (int i = 0; i < (int)is_uniq.size(); ++i) {
    pref_sums[i + 1] = is_uniq[i] + pref_sums[i];
  }

  for (long long len = n; len > k && len + k - 1 >= n; --len) {
    ans += pref_sums[n - len + 1];
  }

  ans += pref_sums[k] * (max<long long>(0, n - k + 1 - k - 1));

  cout << ans << endl;
  return 0;
}